/* $Id: cli.c,v 1.11 2001/07/20 21:42:03 reid Exp $ */

/* This file implements the command line interface. */

#include "lambda.h"
#include "lcore.h"

#include <errno.h>
#include <string.h>


/*** Constants ***/

#define LINE_MAX 512
#define MAX_ARGS 8


/*** Public functions ***/

/* Read and process one line of command input. Returns OK on success, ERR on
   error (and sets the error string), and ERR_EOF on end of file or user exit.

   FIXME [bug 17]: this routine is ugly and should be completely redesigned.
   (The use of goto is not part of this ugliness.) */
lerr cli_process_line(FILE * input)
{
    char line[LINE_MAX];
    char * command;
    char * args[MAX_ARGS];
    int lines, nargs;
    lerr result;
    FILE * fp;
    
    if (fgets(line, LINE_MAX, input) == NULL) /* EOF */
        return ERR_EOF;

    command = strtok(line, " \n");
    if (command == NULL)
        return OK;
    nargs = 0;
    while (nargs < MAX_ARGS
           && (args[nargs] = strtok(NULL, " /\n")) != NULL)
        nargs++;

    if (!strcmp(command, "go")) {
        if (state() == GO)
            goto state_err;
        if (nargs != 0)
            goto args_err;
        return enter_go();
    }
    else if (!strcmp(command, "pause")) {
        if (state() != GO)
            goto state_err;
        if (nargs != 0)
            goto args_err;
        result = enter_pause();
        if (result == OK) {
            timing_stats_pr();
            return OK;
        } else
            return result;
    }
    else if (!strcmp(command, "stop")) {
        if (state() == STOP)
            goto state_err;
        if (nargs != 0)
            goto args_err;
        result = enter_stop(NULL);
        if (result == OK) {
            timing_stats_pr();
            return OK;
        } else
            return result;
    }
    else if (!strcmp(command, "load")) {
        if (state() != STOP)
            goto state_err;
        if (nargs != 2)
            goto args_err;
        return arena_load(args[0], args[1]);
    }
    else if (!strcmp(command, "delete")) {
        if (state() != STOP)
            goto state_err;
        if (nargs != 1)
            goto args_err;
        return arena_delete(args[0]);
    }
    else if (!strcmp(command, "connect")) {
        if (state() != STOP)
            goto state_err;
        if (nargs != 4)
            goto args_err;
        return arena_connect(args[0], args[1], args[2], args[3]);
    }
    else if (!strcmp(command, "disconnect")) {
        if (state() != STOP)
            goto state_err;
        if (nargs != 4)
            goto args_err;
        return arena_disconnect(args[0], args[1], args[2], args[3]);
    }
    else if (!strcmp(command, "set")) {
        /* allowed in all states */
        if (nargs != 3)
            goto args_err;
        return arena_set(args[0], args[1], args[2]);
    }
    else if (!strcmp(command, "unset")) {
        if (state() != STOP)
            goto state_err;
        if (nargs != 2)
            goto args_err;
        return arena_unset(args[0], args[1]);
    }
    else if (!strcmp(command, "print")) {
        /* allowed in all states */
        if (nargs != 0)
            goto args_err;
        return arena_print();
    }
    else if (!strcmp(command, "verify")) {
        if (state() != STOP)
            goto state_err;
        if (nargs != 0)
            goto args_err;
        return arena_ready();
    }
    else if (!strcmp(command, "clear")) {
        if (state() != STOP)
            goto state_err;
        if (nargs != 0)
            goto args_err;
        return arena_clear();
    }
    else if (!strcmp(command, "halt")) {
        err_set("you are not worthy");
        return ERR;
    }
    else if (!strcmp(command, "read")) {
        if (state() != STOP)
            goto state_err;
        if (nargs != 1)
            goto args_err;
        fp = fopen(args[0], "r");
        if (fp == NULL) {
            err_set("can't open %s: %s", args[0], strerror(errno));
            return ERR;
        }
        result = OK;
        lines = 0;
        while (result != ERR_EOF) {
            lines++;
            result = cli_process_line(fp);
            if (result == ERR) {
                err_prepend("%s:%d: ", args[0], lines);
                break;
            }
        }
        if(fclose(fp)) {
            err_set("error closing %s: %s", args[0], strerror(errno));
            return ERR;
        }
        if(result == ERR) {
            /* already set error string */
            return ERR;
        }
        return OK;
    }
    else if (!strcmp(command, "quit")) {
        if (nargs != 0)
            goto args_err;
        if (state() != STOP)
            enter_stop(NULL);
        return ERR_EOF;
    }
    else {
        err_set("unknown command %s", command);
        return ERR;
    }
    
    l_ASSERT(0);
    
args_err:
    err_set("%s: incorrect number of arguments", command);
    return ERR;
state_err:
    err_set("%s: not allowed in %s state", command, state_str());
    return ERR;
}

/* Print the command prompt. */
void cli_prompt(void)
{
    pr("lambda:%5s> ", state_str());
    fflush(stdout);
}
