/* $Id: errors.c,v 1.2 2001/04/20 05:48:27 reid Exp $ */

/* This file contains error handling facilities. Stylistic convention: error
   messages should not begin with a capital letter nor end with a period. If
   it gets too long the error string will be truncated. Do not think about
   pink elephants. */

#include "lambda.h"
#include "lcore.h"

#include <stdarg.h>
#include <stdio.h>
#include <string.h>


/*** Globals ***/

/* The error string. */
char g_err[ERR_CHARS_MAX] = "harvest the microwave burritos";

/* If true, ignore attempts to change the error string. */
bool ignore = 0;


/*** Private prototypes ***/

void err_prepend_nf(char * s);


/*** Public functions ***/

/* Return a pointer to the error string. It is safe to pass the pointer
   returned to other err_*() functions. */
char * err_get(void)
{
    return g_err;
}

/* Return a pointer to a freshly allocated copy of the error string. */
char * err_copy(void)
{
    char * s;

    s = l_malloc(strlen(err_get()) + 1);
    strcpy(s, err_get());

    return s;
}

/* Expand fmt and following arguments according to printf() rules; set the
   error string to the result. */
void err_set(char * fmt, ...)
{
    va_list ap;
    char * s;

    if (ignore)
        return;

    va_start(ap, fmt);
    vasprintf(&s, fmt, ap);
    g_err[0] = '\0';
    err_prepend_nf(s);
    l_free(s);
    va_end(ap);
}


/* Expand fmt and following arguments according to printf() rules; prepend the
   result to the error string. */
void err_prepend(char * fmt, ...)
{
    va_list ap;
    char * s;

    if (ignore)
        return;

    va_start(ap, fmt);
    vasprintf(&s, fmt, ap);
    err_prepend_nf(s);
    l_free(s);
    va_end(ap);
}

/* Ignore any attempts to change the error string until err_ignore_off() is
   called. */
void err_ignore_on(void)
{
    ignore = 1;
}

/* Stop ignoring attempts to change the error string. */
void err_ignore_off(void)
{
    ignore = 0;
}


/*** Private functions ***/

/* Prepend s to the error string. */
void err_prepend_nf(char * s)
{
    size_t delta, count;

    delta = min((int)strlen(s), ERR_CHARS_MAX);
    count = min((int)strlen(g_err) + 1, (int)(ERR_CHARS_MAX - delta - 1));
    memmove(g_err + delta, g_err, count);
    memmove(g_err, s, delta);
    g_err[ERR_CHARS_MAX - 1] = '\0';

    l_ASSERT(g_err[delta + count - 1] == '\0');
}
