/* $Id: memory.c,v 1.5 2001/07/20 21:42:03 reid Exp $ */

/* This file implements lambda's memory management functions. Memory
   allocation and deallocation should be avoided in the go state (because they
   might cause unpredictable delays). */

#include "lambda.h"
#include "lcore.h"

#include <stdlib.h>


/*** Public functions ***/

/* Like malloc(), but cannot fail. Abort the program on failure. */
void * l_malloc(size_t size)
{
    void * p;

    p = malloc(size);
    if (p == NULL) {
        error_pr("\n--\nmalloc(%u) failed\n", size);
        error_pr("
This means FIXME [bug 30].\n");
        abort();
    }

    return p;
}

/* Like realloc(), but cannot fail. Abort the program on failure. */
void * l_realloc(void * p, size_t new_size)
{
    void * new_p;

    new_p = realloc(p, new_size);
    if (new_p == NULL) {
        error_pr("\n--\nrealloc(%p, %u) failed\n", p, new_size);
        error_pr("
This means FIXME [bug 30].\n");
        abort();
    }

    return new_p;
}

/* Like free(). */
void l_free(void * p)
{
    free(p);
}

