/* $Id: osc.c,v 1.2 2001/04/20 05:48:27 reid Exp $ */

#include "lambda.h"
#include "lcore.h"

#include <math.h>


/*** Public functions ***/

/* Allocate a new osc object, with specified duty cycle and starting phase.
   Duty cycle must be less than 1.0. */
osc * osc_new(double phase, double duty)
{
    osc * os;

    l_ASSERT(duty < 1.0);

    os = l_malloc(sizeof(osc));
    os->phase = phase;
    os->duty = duty;

    return os;
}

/* Deallocate the osc object os. */
void osc_free(osc * os)
{
    l_free(os);
}

/* The following functions are oscillators of various waveforms. For each, the
   operation is: Produce count samples using the appropriate waveform at
   frequency f (frequencies which are zero or negative are allowed), storing
   them in buf. Keep state using oscillator os, and it is guaranteed that n
   samples generated with these functions are identical whether they were
   generated by one call or several in turn. It is the caller's responsibility
   to ensure that buf is large enough. */

/* Sine wave oscillator. */
void osc_sine(sample * buf, osc * os, freq f, unsigned count)
{
    unsigned i;
    double step;
   
    step = seconds_per_sample() * f * 2 * M_PI;
    for (i = 0; i < count; i++) {
        buf[i] = sin(os->phase);
        os->phase += step;
        if (os->phase > 2 * M_PI)
            os->phase -= 2 * M_PI;
        else if (os->phase < -2 * M_PI)
            os->phase += 2 * M_PI;
    }
}

