/* $Id: silly-sequencer.c,v 1.1 2001/04/20 05:49:01 reid Exp $ */

#include "lambda.h"


/*** Identification ***/

unsigned magic = MOD_MAGIC_1_0_0;
char name[] = "silly-sequencer";
char version[] = "1.0d1";
char description[] = "A toy sequencer.";


/*** Ports ***/

port_decl port_decls[] = {
    { "f-out", OUT, SCALAR, LTYP_FREQ, 0, STRICT, {} },
};


/*** Types ***/

typedef struct {
    port * out;
} context_t;


/*** Score ***/

#define NOTE_DUR 12
#define NOTE_PCT 1.0
#define TRANSPOSE 0.5

#define C  261.6
#define Cs 277.2
#define D  293.7
#define Ds 311.1
#define E  329.6
#define F  349.2
#define Fs 370.0
#define G  392.0
#define Gs 415.3
#define A  440.0
#define As 466.2
#define B  493.9

#define N1(X) X, X, X, X,  X, X, X, X,  X, X, X, X,  X, X, X, X, \
              X, X, X, X,  X, X, X, X,  X, X, X, X,  X, X, X, X, 0,
#define N2(X) X, X, X, X,  X, X, X, X, \
              X, X, X, X,  X, X, X, X, 0,
#define N4(X) X, X, X, X, \
              X, X, X, X, 0,
#define N8(X) X, X, X, X, 0,
#define N16(X) X, X, 0,
#define N2D(X) X, X, X, X,  X, X, X, X,  X, X, X, X, \
               X, X, X, X,  X, X, X, X,  X, X, X, X, 0,
#define N4D(X) X, X, X, X,  X, X, \
               X, X, X, X,  X, X, 0,
#define N8D(X) X, X, X, X,  X, X, 0,

/* The Star Spangled Banner */
sample score[] = {
N8(G) N8(E) N4(C) N4(E) N4(G) N2(C*2) N8D(E*2) N16(D*2) N4(C*2) N4(E) N4(Fs)
N2(G) N8(G) N8(G) N4D(E*2) N8(D*2) N4(C*2) N2(B) N8(A) N8(B) N4(C*2) N4(C*2)
   N4(G)
N4(E) N4(C) N8D(G) N16(E) N4(C) N4(E) N4(G) N2(C*2) N8D(E*2) N16(D*2)
N4(C*2) N4(E) N4(Fs) N2(G) N8(G) N8(G) N4D(E*2) N8(D*2) N4(C*2)
N2(B) N8(A) N8(B) N4(C*2) N4(C*2) N4(G) N4(E) N4(C) N8(E*2) N8(E*2)
N4(E*2) N4(F*2) N4(G*2) N2(G*2) N8(F*2) N8(E*2) N4(D*2) N4(E*2) N4(F*2)
N2(F*2) N4(F*2) N4D(E*2) N8(D*2) N4(C*2) N2(B) N8(A) N8(B)
N4(C*2) N4(E) N4(Fs) N2(G) N4(G) N4(C*2) N4(C*2) N8(C*2) N8(B)
N4(A) N4(A) N4(A) N4(D*2) N8(F*2) N8(E*2) N8(D*2) N8(C*2) N4(C*2) N2(B) N8(G)
   N8(G)
N4D(C*2) N8(D*2) N8(E*2) N8(F*2) N1(G*2) N8(C*2) N8(D*2) N4D(E*2) N8(F*2)
   N4D(D*2) N2D(C*2)
};


/*** Public functions ***/

lerr lm_go(void ** context, unsigned n_ports, port * ports[])
{
    context_t * x;

    x = l_malloc(sizeof(context_t));
    x->out = port_addrof("f-out", n_ports, ports);
    *context = x;
    return OK;
}

lerr lm_pause(void * context)
{
    return OK;
}

lerr lm_stop(void * context)
{
    l_free(context);
    return OK;
}

lerr lm_tick(void * context)
{
    context_t * x = context;
    freq note = 0.0;

    if (seconds_calc() * NOTE_DUR < sizeof(score) / sizeof(freq))
        note = score[(int)(seconds_calc() * NOTE_DUR)] * TRANSPOSE;

    l_send(x->out, &note);
    return OK;
}
