/* $Id: sin.c,v 1.8 2001/04/20 05:48:27 reid Exp $ */

#include "lambda.h"

#include <math.h>
#include <string.h>


/*** Identification ***/

unsigned magic = MOD_MAGIC_1_0_0;
char name[] = "sin";
char version[] = "1.0d1";
char description[] = "Generates a pure sine wave at the given frequency.";


/*** Port declarations ***/

port_decl port_decls[] = {
    { "out", OUT, ARRAY_REL, LTYP_SAMPLE, 1, STRICT, {} },
    { "f",   IN, SCALAR,    LTYP_FREQ,   0, STRICT, {} },
    END_PORT_DECLARATIONS
};


/*** Types ***/

typedef struct {
    port * out;
    port * f;
    sample * buf;
    osc * os;
} context_t;


/*** Public functions ***/

lerr lm_go(void ** context, unsigned n_ports, port * ports[])
{
    context_t * x;

    x = l_malloc(sizeof(context_t));

    x->out = port_addrof("out", n_ports, ports);
    l_ASSERT(x->out != NULL
             && x->out->direction == OUT
             && port_ready(x->out));
    x->f = port_addrof("f", n_ports, ports);
    l_ASSERT(x->f != NULL
             && x->f->direction == IN
             && port_ready(x->out));

    x->buf = l_malloc(sizeof(sample) * samples_per_tick());
    x->os = osc_new(0.0, 0.0);

    *context = x;
    
    return OK;
}

lerr lm_pause(void * context)
{
    return OK;
}

lerr lm_stop(void * context)
{
    context_t * x = context;

    l_free(x->buf);
    osc_free(x->os);
    l_free(x);

    return OK;
}

lerr lm_tick(void * context)
{
    context_t * x = context;
    freq f;

    l_recv(x->f, &f);
    osc_sine(x->buf, x->os, f, samples_per_tick());
    l_send(x->out, x->buf);

    return OK;
}
