/* $Id: synth-fm.c,v 1.1 2001/04/20 05:49:01 reid Exp $ */

#include "lambda.h"


/*** Identification ***/

unsigned magic = MOD_MAGIC_1_0_0;
char name[] = "synth-fm";
char version[] = "1.0d1";
char description[] = "Simple FM synth; carrier and modulator are sine waves.";


/*** Ports ***/

port_decl port_decls[] = {
    { "f-car", IN,  SCALAR,    LTYP_FREQ,   0, STRICT, {} },
    { "f-mod", IN,  SCALAR,    LTYP_FREQ,   0, STRICT, {} },
    { "index", IN,  SCALAR,    LTYP_DOUBLE, 0, STRICT, {} },
    { "out",   OUT, ARRAY_REL, LTYP_SAMPLE, 1, STRICT, {} }
};


/*** Types ***/

typedef struct {
    port * f_car, * f_mod;
    port * index;
    port * out;
    osc * o_car, * o_mod;
    sample * buf;
} context_t;


/*** Public functions ***/

lerr lm_go(void ** context, unsigned n_ports, port * ports[])
{
    context_t * x;

    x = l_malloc(sizeof(context_t));

    x->f_car = port_addrof("f-car", n_ports, ports);
    x->f_mod = port_addrof("f-mod", n_ports, ports);
    x->index = port_addrof("index", n_ports, ports);
    x->out = port_addrof("out", n_ports, ports);

    x->o_car = osc_new(0.0, 0.0);
    x->o_mod = osc_new(0.0, 0.0);

    x->buf = l_malloc(samples_per_tick() * sizeof(sample));

    *context = x;
    return OK;
}

lerr lm_pause(void * context)
{
    return OK;
}

lerr lm_stop(void * context)
{
    context_t * x = context;

    l_free(x->buf);
    osc_free(x->o_car);
    osc_free(x->o_mod);
    l_free(x);

    return OK;
}

lerr lm_tick(void * context)
{
    context_t * x = context;
    freq f_mod, f_car;
    double index, deviation;
    int i;

    l_recv(x->f_mod, &f_mod);
    l_recv(x->f_car, &f_car);
    l_recv(x->index, &index);
    deviation = index * f_mod;

    osc_sine(x->buf, x->o_mod, f_mod, samples_per_tick());
    for (i = 0; i < samples_per_tick(); i++)
        osc_sine(&(x->buf[i]), x->o_car, f_car + deviation * x->buf[i], 1);

    l_send(x->out, x->buf);
    return OK;
}
